@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #include "log.h"
 #include "BDoc.h"
 #include "Conf.h"
@@ -6,6 +26,7 @@
 #include "crypto/Digest.h"
 #include "util/File.h"
 #include "io/ISerialize.h"
+#include "io/ZipSerialize.h"
 #include "xml/OpenDocument_manifest.hxx"
 
 #include <fstream>
@@ -30,6 +51,17 @@ digidoc::BDoc::BDoc()
 {
 }
 
+/**
+ * Opens BDOC container from a file
+ */
+
+digidoc::BDoc::BDoc(std::string path) throw(IOException, BDocException)
+ : version("1.0")
+{
+    std::auto_ptr<ISerialize> serializer(new ZipSerialize(path));
+    readFrom(serializer);
+}
+
 /**
  * Opens BDOC container using <code>serializer</code> implementation.
  *
@@ -119,7 +151,6 @@ void digidoc::BDoc::save() throw(IOException, BDocException)
     for(std::vector<Document>::const_iterator iter = documents.begin(); iter != documents.end(); iter++)
     {
         serializer->addFile(util::File::fileName(iter->getPath()), iter->getPath());
-        std::cout << "added file " << util::File::fileName(iter->getPath()) << " to documents." << std::endl;
     }
 
     // Add all signatures to container.
@@ -133,6 +164,21 @@ void digidoc::BDoc::save() throw(IOException, BDocException)
     serializer->save();
 }
 
+/**
+ * Saves the container to file using default <code>serializer</code>
+ *
+ * @param path a file to save data to
+ * @throws IOException is thrown if there was a failure saving BDOC container. For example added
+ *         document does not exist.
+ * @throws BDocException is thrown if BDOC class is not correctly initialized.
+ * @see save()
+ */
+void digidoc::BDoc::saveTo(std::string path) throw(IOException, BDocException)
+{
+    std::auto_ptr<ISerialize> serializer(new ZipSerialize(path));
+    saveTo(serializer);
+}
+
 /**
  * Saves the container using the <code>serializer</code> implementation provided.
  *
@@ -636,6 +682,7 @@ void digidoc::BDoc::sign(Signer* signer, Signature::Type profile) throw(BDocExce
         }
         catch(const Exception& e)
         {
+            delete signature;
             THROW_BDOCEXCEPTION_CAUSE(e, "Failed to calculate digests for document '%s'.", iter->getPath().c_str());
         }
     }
@@ -647,10 +694,12 @@ void digidoc::BDoc::sign(Signer* signer, Signature::Type profile) throw(BDocExce
     }
     catch(const SignatureException& e)
     {
+        delete signature;
         THROW_BDOCEXCEPTION_CAUSE(e, "Failed to sign BDOC container.");
     }
     catch(const SignException& e)
     {
+        delete signature;
         THROW_BDOCEXCEPTION_CAUSE(e, "Failed to sign BDOC container.");
     }
 
