@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #include "log.h"
 #include "Conf.h"
 #include "Signature.h"
@@ -74,11 +94,17 @@ digidoc::dsig::SignatureType* digidoc::Signature::createEmptySignature()
 {
     DEBUG("digidoc::Signature::createEmptySignature");
     // Signature->SignedInfo
+
     // Default canonicalization method
-    // Can be either URI_ID_EXC_C14N_NOC or URI_ID_C14N11_NOC
-    // URI_ID_C14N_NOC is not recommended when xml:id attributes are in use
-    dsig::CanonicalizationMethodType canonicalizationMethod(xml_schema::Uri(URI_ID_EXC_C14N_NOC));
-    dsig::SignatureMethodType signatureMethod(xml_schema::Uri("http://www.w3.org/2000/09/xmldsig#rsa-sha1"));//XXX: static const or dynamic
+    // Should be either URI_ID_EXC_C14N_NOC or URI_ID_C14N11_NOC
+    // because URI_ID_C14N_NOC is not recommended when xml:id
+    // attributes are in use ....
+    // http://www.w3.org/TR/DSig-usage chapter 2.1
+    // But the people that created the BDOC-1.0 standard do not 
+    // seem to give a damn about W3C recommendations ...
+    // http://www.signature.lt/-TOOLS/BDoc-1.0.pdf chapter 5.2
+    dsig::CanonicalizationMethodType canonicalizationMethod(xml_schema::Uri(URI_ID_C14N_NOC));
+    dsig::SignatureMethodType signatureMethod(xml_schema::Uri(URI_ID_RSA_SHA1));//XXX: static const or dynamic
     dsig::SignedInfoType signedInfo(canonicalizationMethod, signatureMethod);
 
     // Signature->SignatureValue
@@ -232,7 +258,7 @@ void digidoc::Signature::setSigningCertificate(const X509Cert& x509)
  *
  * @param spp signature production place.
  */
-void digidoc::Signature::setSignatureProductionPlace(const Signer::SignatureProductionPlace& spp)
+void digidoc::Signature::setSignatureProductionPlace(const SignatureProductionPlace& spp)
 {
     DEBUG("setSignatureProductionPlace(spp={city='%s',stateOrProvince='%s',postalCode='%s',countryName='%s'})", spp.city.c_str(), spp.countryName.c_str(), spp.postalCode.c_str(), spp.stateOrProvince.c_str());
 
@@ -252,7 +278,7 @@ void digidoc::Signature::setSignatureProductionPlace(const Signer::SignatureProd
  *
  * @param roles signer roles.
  */
-void digidoc::Signature::setSignerRole(const Signer::SignerRole& roles)
+void digidoc::Signature::setSignerRole(const SignerRole& roles)
 {
     //DEBUG("setSignerRole(signerRole={claimedRoles={'%s'}})", roles.claimedRoles[0].c_str())
 
@@ -371,6 +397,8 @@ std::vector<unsigned char> digidoc::Signature::calcDigestOnNode(Digest* calc, co
         // Set processing flags according to algorithm type.
                if(algorithmType == URI_ID_C14N_NOC) {
             // Default behaviour, nothing needs to be changed
+        } else if(algorithmType == URI_ID_C14N_COM) {
+            canonicalizer.setCommentsProcessing(true);
         } else if(algorithmType == URI_ID_EXC_C14N_NOC) {
 	    // Exclusive mode needs to include xml-dsig in root element 
 	    // in order to maintain compatibility with existing implementations
@@ -378,6 +406,9 @@ std::vector<unsigned char> digidoc::Signature::calcDigestOnNode(Digest* calc, co
 #ifdef URI_ID_C14N11_NOC
         } else if(algorithmType == URI_ID_C14N11_NOC) {
 	    canonicalizer.setInclusive11();
+        } else if(algorithmType == URI_ID_C14N11_COM) {
+	    canonicalizer.setInclusive11();
+            canonicalizer.setCommentsProcessing(true);
 #endif
         } else {
             // Unknown algorithm.
@@ -524,9 +555,9 @@ xml_schema::NamespaceInfomap digidoc::Signature::createNamespaceMap()
  *
  * @return returns structure containing the address of signing place.
  */
-digidoc::Signer::SignatureProductionPlace digidoc::Signature::getProductionPlace() const
+digidoc::SignatureProductionPlace digidoc::Signature::getProductionPlace() const
 {
-    Signer::SignatureProductionPlace productionPlace;
+    SignatureProductionPlace productionPlace;
 
     const xades::SignedSignaturePropertiesType& signedSigProps = getSignedSignatureProperties();
 
@@ -557,9 +588,9 @@ digidoc::Signer::SignatureProductionPlace digidoc::Signature::getProductionPlace
  *
  * @return returns the claimed role of the signer.
  */
-digidoc::Signer::SignerRole digidoc::Signature::getSignerRole() const
+digidoc::SignerRole digidoc::Signature::getSignerRole() const
 {
-    Signer::SignerRole roles;
+    SignerRole roles;
 
     // SignedSignatureProperties
     const xades::SignedSignaturePropertiesType& signedSigProps =
