@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #include <xercesc/dom/DOM.hpp>
 #include <xsec/canon/XSECC14n20010315.hpp>
 #include <xsec/utils/XSECPlatformUtils.hpp>
@@ -41,6 +61,8 @@ digidoc::SignatureBES::SignatureBES(const std::string& path, BDoc& bdoc) throw(S
     checkIfWellFormed();
 }
 
+digidoc::SignatureBES::~SignatureBES() {}
+
 /**
  * @return returns signature mimetype.
  */
@@ -269,7 +291,7 @@ void digidoc::SignatureBES::checkSignatureMethod() const throw(SignatureExceptio
     dsig::SignedInfoType& signedInfo = signature->signedInfo();
     dsig::SignatureMethodType& sigMethod = signedInfo.signatureMethod();
     dsig::SignatureMethodType::AlgorithmType& algorithmType = sigMethod.algorithm();
-    if ( algorithmType != "http://www.w3.org/2000/09/xmldsig#rsa-sha1" )//FIXME: const or dynamic
+    if ( algorithmType != URI_ID_RSA_SHA1 )//FIXME: const or dynamic
     {
         THROW_SIGNATUREEXCEPTION("Unsupported SignedInfo signature method \"%s\"", algorithmType.c_str());
     }
@@ -376,10 +398,9 @@ void digidoc::SignatureBES::checkKeyInfo() const throw(SignatureException)
 	dsig::X509IssuerSerialType::X509IssuerNameType certIssuerName = certs[0].issuerSerial().x509IssuerName();
 	dsig::X509IssuerSerialType::X509SerialNumberType certSerialNumber = certs[0].issuerSerial().x509SerialNumber();
 
-	//XXX: issuer name can be in different formats, compare using OpenSSL xxx_cmp() or write own
 	try
 	{
-		if ( certIssuerName != x509.getIssuerName() || x509.getSerial() != certSerialNumber )
+		if ( x509.compareIssuerToString(certIssuerName) || x509.getSerial() != certSerialNumber )
 		{
 			DEBUG("certIssuerName: \"%s\"", certIssuerName.c_str());
 			DEBUG("x509.getCertIssuerName() \"%s\"", x509.getIssuerName().c_str());
@@ -481,7 +502,7 @@ void digidoc::SignatureBES::checkSigningCertificate() const throw(SignatureExcep
     }
     if(!valid)
     {
-        THROW_SIGNATUREEXCEPTION("Unable to verify signing certificate %s", signingCert.getSubject().c_str());
+        THROW_SIGNATUREEXCEPTION("Unable to verify signing certificate %s", signingCert.getSubjectName().c_str());
     }
 }
 
@@ -525,6 +546,8 @@ const throw(SignatureException)
         THROW_SIGNATUREEXCEPTION("SignedInfo reference to SignedProperties does not have attribute 'URI'");
     }
 
+/*  This check is pointless. It might make sense to check the syntax of the
+ *  URL, but this is better left for the resolvers to handle.
     std::string foundUri = uriOpt.get();
     std::string expectedUri =
         std::string("#") + id() + "-SignedProperties";
@@ -533,6 +556,7 @@ const throw(SignatureException)
     {
         THROW_SIGNATUREEXCEPTION("SignedInfo reference to SignedProperties attribute 'URI' is invalid");
     }
+*/
 
     // check DigestMethod
     const dsig::DigestMethodType& digestMethod = refType.digestMethod();
