@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 /*
  * SignatureTM.cpp
  */
@@ -37,6 +57,9 @@ std::string digidoc::SignatureTM::getMediaType() const
     return MEDIA_TYPE;
 }
 
+/**
+ * @return returns OCSP certificate
+ */
 digidoc::X509Cert digidoc::SignatureTM::getOCSPCertificate() const
 {
     const xades::UnsignedSignaturePropertiesType::CertificateValuesType::EncapsulatedX509CertificateSequence &certs =
@@ -58,6 +81,9 @@ digidoc::X509Cert digidoc::SignatureTM::getOCSPCertificate() const
     return X509Cert();
 }
 
+/**
+ * @return returns OCSP timestamp
+ */
 std::string digidoc::SignatureTM::getProducedAt() const
 {
     const xades::OCSPIdentifierType::ProducedAtType &producedAt =
@@ -66,6 +92,9 @@ std::string digidoc::SignatureTM::getProducedAt() const
     return util::date::xsd2string(producedAt);
 }
 
+/**
+ * @return returns OCSP responder ID
+ */
 std::string digidoc::SignatureTM::getResponderID() const
 {
     return unsignedSignatureProperties()->completeRevocationRefs()[0].oCSPRefs()
@@ -107,8 +136,10 @@ void digidoc::SignatureTM::validateOffline() const throw(SignatureException)
     }
     catch( const Exception &e )
     {
-        THROW_SIGNATUREEXCEPTION_CAUSE( e, "OCSP certificate loading failed" );
-    }
+        SignatureException exception(__FILE__, __LINE__, "OCSP certificate loading failed", e);
+        exception.setCode( Exception::OCSPCertMissing );
+        throw exception;
+	}
     X509Stack_scope x509StackScope(&ocspCerts);
     ocsp.setOCSPCerts(ocspCerts);
     //ocsp.setCertStore(digidoc::X509CertStore::getInstance()->getCertStore());
@@ -192,11 +223,11 @@ void digidoc::SignatureTM::sign(Signer* signer) throw(SignatureException, SignEx
     if(issuer == NULL)
     {
     	THROW_SIGNATUREEXCEPTION("Could not find certificate '%s' issuer '%s' in certificate store.",
-    			cert_.getSubject().c_str(), cert_.getIssuerName().c_str());
+    			cert_.getSubjectName().c_str(), cert_.getIssuerName().c_str());
     }
 
     DEBUG("Signing with X.509 cert {serial=%ld, subject=%s, issuer=%s})",
-    		cert_.getSerial(), cert_.getSubject().c_str(), cert_.getIssuerName().c_str());
+    		cert_.getSerial(), cert_.getSubjectName().c_str(), cert_.getIssuerName().c_str());
 
     // Initialize OCSP.
     DEBUG("Making OCSP request.");
@@ -387,7 +418,7 @@ void digidoc::SignatureTM::setCACertificate(const X509Cert& x509)
  */
 void digidoc::SignatureTM::addCertificateValue(const std::string& certId, const X509Cert& x509)
 {
-    DEBUG("digidoc::SignatureTM::setCertificateValue(%s, X509Cert{%ld,%s})", certId.c_str(), x509.getSerial(), x509.getSubject().c_str());
+    DEBUG("digidoc::SignatureTM::setCertificateValue(%s, X509Cert{%ld,%s})", certId.c_str(), x509.getSerial(), x509.getSubjectName().c_str());
     //CertificateValues
     std::vector<unsigned char> certBytes = x509.encodeDER();
     xades::CertificateValuesType::EncapsulatedX509CertificateType certData(
