@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #if defined(_WIN32)
 // for converting between DOS and Windows code pages with OemToChar()
 #include <windows.h> 
@@ -503,8 +523,8 @@ public:
 
 		//Signer* signer;
 		Signature::Type profile;
-		Signer::SignatureProductionPlace spp( argv[3], argv[4], argv[5], argv[6] );
-		Signer::SignerRole role( argv[7] );
+		SignatureProductionPlace spp( argv[3], argv[4], argv[5], argv[6] );
+		SignerRole role( argv[7] );
 
 
 		if ( std::string( "TM" ) == argv[2] )
@@ -592,7 +612,7 @@ public:
 		// type
 		std::cout << "Signature " << i << " (" << sig->getMediaType().c_str() << ")" << std::endl;
 		// production place
-		Signer::SignatureProductionPlace spp = sig->getProductionPlace();
+		SignatureProductionPlace spp = sig->getProductionPlace();
 		if ( !spp.isEmpty() )
 		{
 			std::cout << "  Signature production place:" << std::endl;
@@ -603,11 +623,11 @@ public:
 		}
 
 		// signer role
-		Signer::SignerRole roles = sig->getSignerRole();
+		SignerRole roles = sig->getSignerRole();
 		if ( !roles.isEmpty() )
 		{
 			std::cout << "  Signer role(s):" << std::endl;
-			for ( Signer::SignerRole::TRoles::const_iterator iter = roles.claimedRoles.begin(); iter != roles.claimedRoles.end(); iter++ )
+			for ( SignerRole::TRoles::const_iterator iter = roles.claimedRoles.begin(); iter != roles.claimedRoles.end(); iter++ )
 			{
 				std::cout << "    " << iter->c_str() << std::endl;
 			}
