@@ -1,12 +1,39 @@
-#include <openssl/bio.h>
-#include <openssl/pem.h>
-#include <openssl/err.h>
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "X509CertStore.h"
+
 #include "../../log.h"
 #include "../../crypto/OpenSSLHelpers.h"
 #include "../../util/String.h"
-#include "X509Cert.h"
-#include "X509CertStore.h"
 
+#include <sstream>
+#include <openssl/opensslconf.h>
+#include <openssl/opensslv.h>
+#include <openssl/bio.h>
+#include <openssl/pem.h>
+#include <openssl/err.h>
+
+/**
+ * Creates NULL X509 certificate.
+ */
 digidoc::X509Cert::X509Cert(): cert(NULL) {}
 
 /**
@@ -36,7 +63,11 @@ digidoc::X509Cert::X509Cert(std::vector<unsigned char> bytes) throw(IOException)
     }
 
     // Parse certificate from DER formatted buffer.
+#if OPENSSL_VERSION_NUMBER >= 0x0090800fL
     const unsigned char* pBytes = reinterpret_cast<const unsigned char*>(&bytes[0]);
+#else
+    unsigned char* pBytes = &bytes[0];
+#endif 
     d2i_X509(&cert, &pBytes, bytes.size());
     if(cert == NULL)
     {
@@ -279,7 +310,7 @@ std::string digidoc::X509Cert::getIssuerName() const throw(IOException)
  * @return issuer name converted to string.
  * @throws IOException exception is throws if the conversion failed.
  */
-std::string digidoc::X509Cert::getSubject() const throw(IOException)
+std::string digidoc::X509Cert::getSubjectName() const throw(IOException)
 {
     X509_NAME* subject = X509_get_subject_name(cert);
     if(subject == NULL)
@@ -290,6 +321,70 @@ std::string digidoc::X509Cert::getSubject() const throw(IOException)
     return toString(subject);
 }
 
+/**
+ * Converts individual X.509 subject entry to string.
+ *
+ * @param ln Long name of the requested entry (see openssl/objects.h)
+ * @return subject name entry converted to string.
+ * @throws IOException exception is throws if the conversion failed.
+ */
+std::string digidoc::X509Cert::getSubjectInfo(const std::string& ln) const throw(IOException)
+{
+    X509_NAME* name = X509_get_subject_name(cert);
+    if(name == NULL)
+    {
+       THROW_IOEXCEPTION("Failed to convert X.509 certificate subject: %s", ERR_reason_error_string(ERR_get_error()));
+    }
+    int idx = X509_NAME_get_index_by_NID(name, OBJ_ln2nid(ln.c_str()), -1);
+    if(idx < 0)
+    {
+       THROW_IOEXCEPTION("Failed to retrieve X.509 certificate info: field '%s' not found", ln.c_str());
+    }
+    X509_NAME_ENTRY *entry = X509_NAME_get_entry(name, idx);
+    if(!entry)
+    {
+       THROW_IOEXCEPTION("Failed to convert X.509 certificate field: %s", ERR_reason_error_string(ERR_get_error()));
+    }
+    ASN1_STRING *asnstr = X509_NAME_ENTRY_get_data(entry);
+
+    unsigned char *data = NULL;
+    int len = ASN1_STRING_to_UTF8(&data, asnstr);
+    if(len < 0)
+    {
+       THROW_IOEXCEPTION("Failed to convert X.509 certificate field: %s", ERR_reason_error_string(ERR_get_error()));
+    }
+    std::string result(reinterpret_cast<const char *>(data));
+    OPENSSL_free(data);
+
+    return result;
+}
+
+
+/**
+ * Converts individual X.509 issuer entry to string.
+ *
+ * @param ln long name of the requested entry (see openssl/objects.h)
+ * @return issuer name entry converted to string.
+ * @throws IOException exception is throws if the conversion failed.
+ */
+std::string digidoc::X509Cert::getIssuerInfo(const std::string& ln) const throw(IOException)
+{
+    char buf[1024];
+
+    X509_NAME* name = X509_get_issuer_name(cert);
+    if(name == NULL)
+    {
+       THROW_IOEXCEPTION("Failed to convert X.509 certificate issuer: %s", ERR_reason_error_string(ERR_get_error()));
+    }
+
+    if(X509_NAME_get_text_by_NID(name, OBJ_ln2nid(ln.c_str()), buf, 1024) < 0)
+    {
+       THROW_IOEXCEPTION("Failed to retrieve X.509 certificate info: field '%s' not found", ln.c_str());
+    }
+
+    return std::string(buf);
+}
+
 /**
  * Converts X509_NAME struct to string.
  *
@@ -332,6 +427,7 @@ EVP_PKEY* digidoc::X509Cert::getPublicKey() const throw(IOException)
     EVP_PKEY* pubKey = X509_get_pubkey(cert);
     if((pubKey == NULL) || (pubKey->type != EVP_PKEY_RSA))
     {
+        EVP_PKEY_free(pubKey);
         THROW_IOEXCEPTION("Unable to load RSA public Key: %s", ERR_reason_error_string(ERR_get_error()));
     }
     return pubKey;
@@ -350,6 +446,7 @@ std::vector<unsigned char> digidoc::X509Cert::getRsaModulus() const throw(IOExce
 
     if(bufSize <= 0)
     {
+        EVP_PKEY_free(pubKey);
         THROW_IOEXCEPTION("Failed to extract RSA modulus.");
     }
 
@@ -359,10 +456,12 @@ std::vector<unsigned char> digidoc::X509Cert::getRsaModulus() const throw(IOExce
     // Extract RSA modulus.
     if(BN_bn2bin(pubKey->pkey.rsa->n, &rsaModulus[0]) <= 0)
     {
+        EVP_PKEY_free(pubKey);
         THROW_IOEXCEPTION("Failed to extract RSA modulus.");
     }
 
-	return rsaModulus;
+    EVP_PKEY_free(pubKey);
+    return rsaModulus;
 }
 
 /**
@@ -378,6 +477,7 @@ std::vector<unsigned char> digidoc::X509Cert::getRsaExponent() const throw(IOExc
 
     if(bufSize <= 0)
     {
+        EVP_PKEY_free(pubKey);
         THROW_IOEXCEPTION("Failed to extract RSA exponent.");
     }
 
@@ -387,10 +487,12 @@ std::vector<unsigned char> digidoc::X509Cert::getRsaExponent() const throw(IOExc
     // Extract RSA exponent.
     if(BN_bn2bin(pubKey->pkey.rsa->e, &rsaExponent[0]) <= 0)
     {
+        EVP_PKEY_free(pubKey);
         THROW_IOEXCEPTION("Failed to extract RSA exponent.");
     }
 
-	return rsaExponent;
+    EVP_PKEY_free(pubKey);
+    return rsaExponent;
 }
 
 bool digidoc::X509Cert::isValid() const throw(IOException)
@@ -435,11 +537,33 @@ int digidoc::X509Cert::verify(X509_STORE* aStore) const throw(IOException)
 
     if(!ok)
     {
-       int err  = X509_STORE_CTX_get_error(csc);
-
-       X509Cert cause(X509_STORE_CTX_get_current_cert (csc));
-       ERR("Unable to verify %s. Cause: %s", cause.getSubject().c_str(), X509_verify_cert_error_string(err));
+        int err = X509_STORE_CTX_get_error(csc);
+        X509Cert cause(X509_STORE_CTX_get_current_cert (csc));
+        std::ostringstream s;
+        s << "Unable to verify " << cause.getSubjectName();
+        s << ". Cause: " << X509_verify_cert_error_string(err);
+        switch(err)
+        {
+        case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
+        {
+            IOException e(__FILE__, __LINE__, s.str());
+            e.setCode( Exception::CertificateIssuerMissing );
+            throw e;
+            break;
+        }
+        default: THROW_IOEXCEPTION(s.str().c_str()); break;
+        }
     }
 
     return ok;
 }
+
+int digidoc::X509Cert::compareIssuerToString(const std::string& in) const throw(IOException) {
+    //FIXME: Actually implement this check
+    //http://wiki.ngs.ac.uk/index.php?title=Comparing_DNs
+
+    //retval = X509_NAME_cmp(this->getIssuerNameAsn1(), xn);
+    //X509_NAME_free(xn);
+
+    return 0;
+}
