@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #if defined(_WIN32)
 #include <conio.h>
 #endif
@@ -40,8 +60,8 @@ digidoc::EstEIDSigner::~EstEIDSigner()
  * @return should return the selected certificate.
  * @throws SignException throws exception if no suitable certificate was found.
  */
-digidoc::PKCS11SignerAbstract::PKCS11Cert digidoc::EstEIDSigner::selectSigningCertificate(
-        std::vector<PKCS11SignerAbstract::PKCS11Cert> certificates) throw(SignException)
+digidoc::PKCS11Signer::PKCS11Cert digidoc::EstEIDSigner::selectSigningCertificate(
+        std::vector<PKCS11Signer::PKCS11Cert> certificates) throw(SignException)
 {
     // Find EstEID signing certificate
     if(certificates.empty())
@@ -100,7 +120,7 @@ void digidoc::EstEIDConsolePinSigner::setPrompt(const std::string& prompt)
  * @return returns PIN code.
  * @throws SignException throws an exception if the login process is canceled.
  */
-std::string digidoc::EstEIDConsolePinSigner::getPin(PKCS11SignerAbstract::PKCS11Cert certificate) throw(SignException)
+std::string digidoc::EstEIDConsolePinSigner::getPin(PKCS11Signer::PKCS11Cert certificate) throw(SignException)
 {
     char pin[16];
     size_t pinMax = 16;
