@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #include <memory.h>
 #include <map>
 #include <libp11.h>
@@ -22,7 +42,6 @@ struct SignSlot
 {
 	PKCS11_CERT* certificate;
 	PKCS11_SLOT* slot;
-	int			number;
 };
 
 class PKCS11SignerPrivate
@@ -36,10 +55,9 @@ public:
 	{
 		sign.certificate = NULL;
 		sign.slot = NULL;
-		sign.number = -1;
 	};
 
-    digidoc::PKCS11SignerAbstract::PKCS11Cert createPKCS11Cert(PKCS11_SLOT* slot, PKCS11_CERT* cert);
+    digidoc::PKCS11Signer::PKCS11Cert createPKCS11Cert(PKCS11_SLOT* slot, PKCS11_CERT* cert);
     bool checkCert( X509 *cert ) const;
 
     std::string driver;
@@ -91,8 +109,6 @@ digidoc::PKCS11Signer::~PKCS11Signer()
 	delete d;
 }
 
-int digidoc::PKCS11Signer::slotNumber() const { return d->sign.number; }
-
 void digidoc::PKCS11Signer::unloadDriver()
 {
     if(d->ctx == NULL)
@@ -168,7 +184,6 @@ X509* digidoc::PKCS11Signer::getCert() throw(SignException)
     // Set selected state to 'no certificate selected'.
     d->sign.certificate = NULL;
     d->sign.slot = NULL;
-    d->sign.number = -1;
     if(d->slots != NULL)
     {
         // Release all slots.
@@ -182,9 +197,8 @@ X509* digidoc::PKCS11Signer::getCert() throw(SignException)
     }
 
     // Iterate over all found slots, if the slot has a token, check if the token has any certificates.
-    std::vector<PKCS11SignerAbstract::PKCS11Cert> certificates;
+    std::vector<PKCS11Signer::PKCS11Cert> certificates;
     std::vector<SignSlot> certSlotMapping;
-    int tokenId = 0;
     for(unsigned int i = 0; i < d->numberOfSlots; i++)
     {
         PKCS11_SLOT* slot = d->slots + i;
@@ -212,11 +226,10 @@ X509* digidoc::PKCS11Signer::getCert() throw(SignException)
                 PKCS11_CERT* cert = certs + j;
                 if(!d->checkCert(cert->x509))
                     break;
-                SignSlot signSlot = { cert, slot, tokenId };
+                SignSlot signSlot = { cert, slot };
                 certSlotMapping.push_back( signSlot );
                 certificates.push_back(d->createPKCS11Cert(slot, cert));
             }
-            ++tokenId;
         }
     }
 
@@ -334,6 +347,10 @@ void digidoc::PKCS11Signer::sign(const Digest& digest, Signature& signature) thr
     }
 }
 
+/**
+ * @param cert X509 certificate
+ * @return returns if certificate is valid for signing
+ */
 bool digidoc::PKCS11SignerPrivate::checkCert( X509 *cert ) const
 {
     if(!cert)
@@ -362,9 +379,9 @@ bool digidoc::PKCS11SignerPrivate::checkCert( X509 *cert ) const
  * @param cert cert to be used to init PKCS11Cert.
  * @return returns created PKCS11Cert struct.
  */
-digidoc::PKCS11SignerAbstract::PKCS11Cert digidoc::PKCS11SignerPrivate::createPKCS11Cert(PKCS11_SLOT* slot, PKCS11_CERT* cert)
+digidoc::PKCS11Signer::PKCS11Cert digidoc::PKCS11SignerPrivate::createPKCS11Cert(PKCS11_SLOT* slot, PKCS11_CERT* cert)
 {
-    digidoc::PKCS11SignerAbstract::PKCS11Cert certificate;
+    digidoc::PKCS11Signer::PKCS11Cert certificate;
     certificate.token.label = slot->token->label;
     certificate.token.manufacturer = slot->token->manufacturer;
     certificate.token.model = slot->token->model;
