@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #if !defined(__PKCS11SIGNER_H_INCLUDED__)
 #define __PKCS11SIGNER_H_INCLUDED__
 
@@ -5,21 +25,6 @@
 
 namespace digidoc
 {
-    class EXP_DIGIDOC PKCS11SignerAbstract: public Signer
-    {
-      public:
-        enum { Type = 1 };
-        struct PKCS11Token { std::string label, manufacturer, model, serialNr; };
-        struct PKCS11Cert { PKCS11Token token; std::string label; X509* cert; };
-
-        virtual X509* getCert() throw(SignException) = 0;
-        virtual std::string getPin(PKCS11Cert certificate) throw(SignException) = 0;
-        virtual void loadDriver() throw(SignException) = 0;
-        virtual void loadDriver(const std::string& driver) throw(SignException) = 0;
-        virtual void unloadDriver() = 0;
-        virtual int  slotNumber() const = 0;
-        virtual int  type() { return Type; }
-    };
 
     /**
      * Implements <code>Signer</code> interface for ID-Cards, which support PKCS #11 protocol.
@@ -38,10 +43,13 @@ namespace digidoc
      * @author Janari Põld
      */
 	class PKCS11SignerPrivate;
-    class EXP_DIGIDOC PKCS11Signer : public PKCS11SignerAbstract
+    class EXP_DIGIDOC PKCS11Signer : public Signer
     {
 
       public:
+          struct PKCS11Token { std::string label, manufacturer, model, serialNr; };
+          struct PKCS11Cert { PKCS11Token token; std::string label; X509* cert; };
+
           PKCS11Signer() throw(SignException);
           PKCS11Signer(const std::string& driver) throw(SignException);
           virtual ~PKCS11Signer();
@@ -51,7 +59,6 @@ namespace digidoc
           void loadDriver() throw(SignException);
           void loadDriver(const std::string& driver) throw(SignException);
           void unloadDriver();
-          int  slotNumber() const;
 
           /**
            * Abstract method that returns PIN code for the selected signing certificate.
@@ -64,7 +71,7 @@ namespace digidoc
            * @throws SignException should throw an exception if the login operation
            *         should be canceled.
            */
-          virtual std::string getPin(PKCS11SignerAbstract::PKCS11Cert certificate) throw(SignException) = 0;
+          virtual std::string getPin(PKCS11Cert certificate) throw(SignException) = 0;
 
       protected:
 
@@ -78,7 +85,7 @@ namespace digidoc
            * @throws SignException should throw an exception if no suitable certificate
            *         is in the list or the operation should be canceled.
            */
-          virtual PKCS11SignerAbstract::PKCS11Cert selectSigningCertificate(std::vector<PKCS11SignerAbstract::PKCS11Cert> certificates) throw(SignException) = 0;
+          virtual PKCS11Cert selectSigningCertificate(std::vector<PKCS11Cert> certificates) throw(SignException) = 0;
 
           virtual void showPinpad() {};
           virtual void hidePinpad() {};
