@@ -1,3 +1,23 @@
+/*
+ * libdigidocpp - a C++ library for creating and validating BDOC-1.0 documents
+ *
+ * Copyright (C) 2009-2010  Estonian Informatics Centre
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 
 // This file has platform-specific implementations.
 // Treat all POSIX systems (Linux, MAC) the same way. Treat non-POSIX as Windows.
@@ -35,12 +55,27 @@
 
 std::stack<std::string> digidoc::util::File::tempFiles;
 
+/**
+ * Encodes path to compatible std lib
+ * @param fileName path
+ * @return encoded path
+ */
 std::string digidoc::util::File::encodeName(const std::string &fileName)
 { return digidoc::util::String::convertUTF8(fileName,false); }
 
+/**
+ * Decodes path from std lib path
+ * @param localFileName path
+ * @return decoded path
+ */
 std::string digidoc::util::File::decodeName(const std::string &localFileName)
 { return digidoc::util::String::convertUTF8(localFileName,true); }
 
+/**
+ * Encodes path to compatible std fstream
+ * @param fileName path
+ * @return encoded path
+ */
 #if _WIN32
 std::wstring digidoc::util::File::fstreamName(const std::string &fileName)
 { return digidoc::util::String::toWideChar(CP_UTF8, fileName); }
@@ -612,12 +647,12 @@ void digidoc::util::File::removeDirectoryRecursively(const std::string& dname) t
     std::vector<std::string> subFiles = getDirSubElements(dname, false, true, false);
     for (std::vector<std::string>::reverse_iterator it = subFiles.rbegin(); it != subFiles.rend(); it++)
     {
-        WARN( "Deleting the temporary file '%s'", it->c_str() );
+        DEBUG( "Deleting the temporary file '%s'", it->c_str() );
         removeFile(*it);
     }
 
     // Then delete the directory itself. It should now be empty.
-    WARN( "Deleting the temporary directory '%s'", dname.c_str() );
+    DEBUG( "Deleting the temporary directory '%s'", dname.c_str() );
     removeDirectory(dname);
 }
 
